@@ -22,9 +22,10 @@ typedef std::set<pkgCache::PkgIterator> pkgset;
 typedef std::vector<string> strvector;
 
 enum cmdline_pkgaction_type
-  {cmdline_install, cmdline_remove, cmdline_purge, cmdline_hold,
-   cmdline_unhold, cmdline_markauto, cmdline_unmarkauto,
-   cmdline_forbid_version, cmdline_reinstall};
+  {cmdline_install, cmdline_installauto, cmdline_remove,
+   cmdline_purge, cmdline_hold, cmdline_unhold, cmdline_markauto,
+   cmdline_unmarkauto, cmdline_forbid_version, cmdline_reinstall,
+   cmdline_keep};
 
 enum cmdline_version_source {cmdline_version_cand,
 			     cmdline_version_archive,
