@@ -6,15 +6,18 @@
 
 #include "cmdline_action.h"
 #include "cmdline_changelog.h"
+#include "cmdline_resolver.h"
 #include "cmdline_show.h"
 #include "cmdline_util.h"
 
-#include <download_manager.h>
 #include <ui.h>
 
-#include <generic/apt.h>
-#include <generic/infer_reason.h>
+#include <generic/apt/apt.h>
+#include <generic/apt/config_signal.h>
+#include <generic/apt/download_signal_log.h>
+#include <generic/apt/infer_reason.h>
 
+#include <vscreen/fragment.h>
 #include <vscreen/vscreen.h>
 
 #include <apt-pkg/algorithms.h>
@@ -25,6 +28,11 @@
 
 using namespace std;
 
+string StdinEOFException::errmsg() const
+{
+  return _("Unexpected end-of-file on standard input");
+}
+
 struct fetchinfo
 {
   double FetchBytes, FetchPBytes, DebBytes;
@@ -37,7 +45,7 @@ static bool pkg_byname_compare(const pkgCache::PkgIterator &a, const pkgCache::P
 
 static bool get_fetchinfo(fetchinfo &f)
 {
-  download_manager m;
+  download_signal_log m;
   pkgAcquire fetcher(&m);
   pkgSourceList l;
   if(!l.ReadMainList())
@@ -98,7 +106,8 @@ static string reason_string_list(set<reason> &reasons)
 static void cmdline_show_instinfo(pkgvector &items,
 				  bool showvers,
 				  bool showdeps,
-				  bool showsize)
+				  bool showsize,
+				  bool showpurge)
 {
   sort(items.begin(), items.end(), pkg_byname_compare);
   strvector output;
@@ -111,6 +120,12 @@ static void cmdline_show_instinfo(pkgvector &items,
       //aptitudeDepCache::aptitude_state &extstate=(*apt_cache_file)->get_ext_state(*i);
       pkgCache::VerIterator instver=state.InstVerIter(*apt_cache_file);
 
+      if(showpurge)
+	{
+	  if(state.Delete() && state.iFlags&pkgDepCache::Purge)
+	    s += "{p}";
+	}
+
       // Display version numbers.
       if(showvers)
 	{
@@ -260,19 +275,25 @@ static const char *cmdline_action_descriptions[num_pkg_action_states]={
 
 // Probably something like cin.getline() would work, but I don't trust that
 // for interactive use.
-static string prompt_string()
+string prompt_string(const string &prompt)
 {
+  printf("%s", prompt.c_str());
+  fflush(stdout);
+
   string rval;
   char buf[1024];
   cin.getline(buf, 1023);
   rval+=buf;
 
-  while(!cin)
+  while(!cin && !cin.eof())
     {
       cin.getline(buf, 1023);
       rval+=buf;
     }
 
+  if(!cin)
+    throw StdinEOFException();
+
   return rval;
 }
 
@@ -332,16 +353,18 @@ static bool prompt_essential()
       cin.getline(buf, 1023);
       bool rval=(prompt==buf);
 
-      while(!cin)
+      while(!cin && !cin.eof())
 	cin.getline(buf, 1023);
 
+      if(!cin)
+	throw StdinEOFException();
+
       return rval;
     }
 
   return true;
 }
 
-#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
 /** Checks for trust violations and displays a big fat warning if any exist.
  *
  *  \return true if everything is OK or the user overrode the warning.
@@ -375,16 +398,31 @@ static bool prompt_trust()
 
       cmdline_show_pkglist(untrusted);
 
+      printf("\n");
+
+
+      if(aptcfg->FindB(PACKAGE "::CmdLine::Ignore-Trust-Violations", false))
+	{
+	  printf(_("*** WARNING ***   Ignoring these trust violations because\n"
+		   "                  %s::CmdLine::Ignore-Trust-Violations is 'true'!\n"),
+		 PACKAGE);
+	  return true;
+	}
+
+
       const string okstr=_("Yes"), abortstr=_("No");
 
       while(1)
 	{
-	  printf(_("Should I go ahead and install the packages anyway?\n"));
+	  printf(_("Do you want to ignore this warning and proceed anyway?\n"));
 	  printf(_("To continue, enter \"%s\"; to abort, enter \"%s\": "), okstr.c_str(), abortstr.c_str());
 	  char buf[1024];
 	  cin.getline(buf, 1023);
 	  buf[1023]='\0';
 
+	  if(cin.eof())
+	    throw StdinEOFException();
+
 	  const bool is_ok=(strncasecmp(okstr.c_str(), buf, okstr.size())==0);
 	  const bool is_abort=(strncasecmp(abortstr.c_str(), buf, abortstr.size())==0);
 
@@ -399,12 +437,6 @@ static bool prompt_trust()
 
   return true;
 }
-#else
-static bool prompt_trust()
-{
-  return true;
-}
-#endif
 
 /** Displays a preview of the stuff to be done -- like apt-get, it collects
  *  all the "stuff to install" in one place.
@@ -487,23 +519,26 @@ static bool cmdline_show_preview(bool as_upgrade, pkgset &to_install,
 	  if(i==pkg_auto_install || i==pkg_auto_remove || i==pkg_unused_remove ||
 	     i==pkg_auto_hold || i==pkg_broken)
 	    cmdline_show_instinfo(lists[i],
-				  showvers, showdeps, showsize);
+				  showvers, showdeps, showsize,
+				  (i == pkg_auto_remove ||
+				   i == pkg_unused_remove));
 	  else
 	    cmdline_show_instinfo(lists[i],
-				  showvers, false, showsize);
+				  showvers, false, showsize,
+				  i == pkg_remove);
 	}
     }
 
   if(!recommended.empty())
     {
       printf(_("The following packages are RECOMMENDED but will NOT be installed:\n"));
-      cmdline_show_instinfo(recommended, showvers, showdeps, showsize);
+      cmdline_show_instinfo(recommended, showvers, showdeps, showsize, false);
     }
 
   if(verbose>0 && !suggested.empty())
     {
       printf(_("The following packages are SUGGESTED but will NOT be installed:\n"));
-      cmdline_show_instinfo(suggested, showvers, showdeps, showsize);
+      cmdline_show_instinfo(suggested, showvers, showdeps, showsize, false);
     }
 
   if(all_empty)
@@ -550,89 +585,6 @@ static bool cmdline_show_preview(bool as_upgrade, pkgset &to_install,
   return rval;
 }
 
-// Parses a list of actions and does them, running a fixer afterwards.
-// Aborts after a partial action if something bad happens.
-static void cmdline_parse_action(string s,
-				 pkgset &to_install, pkgset &to_hold,
-				 pkgset &to_remove, pkgset &to_purge,
-				 int verbose)
-{
-  string::size_type loc=0;
-  pkgProblemResolver fixer(*apt_cache_file);
-
-  while(loc<s.size() && isspace(s[loc]))
-    ++loc;
-
-  if(loc==s.size())
-    return;
-
-  cmdline_pkgaction_type action;
-
-  switch(s[loc])
-    {
-    case '_':
-      action=cmdline_purge;
-      break;
-    case '-':
-      action=cmdline_remove;
-      break;
-    case '=':
-      action=cmdline_hold;
-      break;
-    case '+':
-      action=cmdline_install;
-      break;
-    default:
-      printf(_("Bad action character '%c'\n"), s[0]);
-      return;
-    }
-
-  ++loc;
-
-  while(loc<s.size())
-    {
-      while(loc<s.size() && isspace(s[loc]))
-	++loc;
-
-      if(loc<s.size())
-	switch(s[loc])
-	  {
-	  case '_':
-	    action=cmdline_purge;
-	    ++loc;
-	    break;
-	  case '-':
-	    action=cmdline_remove;
-	    ++loc;
-	    break;
-	  case '=':
-	    action=cmdline_hold;
-	    ++loc;
-	    break;
-	  case '+':
-	    action=cmdline_install;
-	    ++loc;
-	    break;
-	  default:
-	    {
-	      string pkgname;
-	      while(loc<s.size() && !isspace(s[loc]))
-		pkgname+=s[loc++];
-
-	      if(!cmdline_applyaction(pkgname, action, fixer,
-				      to_install, to_hold,
-				      to_remove, to_purge,
-				      verbose))
-		return;
-	    }
-	  }
-    }
-
-  // Er, what do we do here?
-  if(!(*apt_cache_file)->try_fix_broken(fixer, NULL))
-    _error->DumpErrors();
-}
-
 static void cmdline_parse_show(string response,
 			       int verbose)
 {
@@ -660,8 +612,7 @@ static void cmdline_parse_show(string response,
   if(!one_shown)
     printf(_("No packages to show -- enter the package names on the line after 'i'.\n"));
 
-  printf(_("Press Return to continue."));
-  prompt_string();
+  prompt_string(_("Press Return to continue."));
 }
 
 // Erm.  Merge w/ above?
@@ -691,8 +642,76 @@ static void cmdline_parse_changelog(string response)
   else
     do_cmdline_changelog(packages);
 
-  printf(_("Press Return to continue."));
-  prompt_string();
+  prompt_string(_("Press Return to continue"));
+}
+
+static inline fragment *flowindentbox(int i1, int irest, fragment *f)
+{
+  return indentbox(i1, irest, flowbox(f));
+}
+
+static void prompt_help(ostream &out)
+{
+  fragment *f=indentbox(2, 2,
+			fragf(_("y: %F"
+				"n: %F"
+				"i: %F"
+				"c: %F"
+				"d: %F"
+				"s: %F"
+				"v: %F"
+				"e: %F"
+				"%n"
+				"%F"
+				"%n"
+				"%F"
+				"%F"
+				"%F"
+				"%F"
+				"%F"
+				"%F"
+				"%F"
+				"%F"),
+			      flowindentbox(0, 3,
+					    fragf(_("continue with the installation"))),
+			      flowindentbox(0, 3,
+					    fragf(_("abort and quit"))),
+			      flowindentbox(0, 3,
+					    fragf(_("show information about one or more packages; the package names should follow the 'i'"))),
+			      flowindentbox(0, 3,
+					    fragf(_("show the Debian changelogs of one or more packages; the package names should follow the 'c'"))),
+			      flowindentbox(0, 3,
+					    fragf(_("toggle the display of dependency information"))),
+			      flowindentbox(0, 3,
+					    fragf(_("toggle the display of changes in package sizes"))),
+			      flowindentbox(0, 3,
+					    fragf(_("toggle the display of version numbers"))),
+			      flowindentbox(0, 3,
+					    fragf(_("enter the full visual interface"))),
+			      flowbox(fragf(_("You may also specify modification to the actions which will be taken.  To do so, type an action character followed by one or more package names (or patterns).  The action will be applied to all the packages that you list.  The following actions are available:"))),
+			      // FIXME: copied from
+			      // cmdline_resolver.cc, maybe this
+			      // should be placed in a common file?
+			      flowindentbox(0, 4,
+					    fragf(_("'+' to install packages"))),
+			      flowindentbox(0, 5,
+					    fragf(_("'+M' to install packages and immediately flag them as automatically installed"))),
+			      flowindentbox(0, 4,
+					    fragf(_("'-' to remove packages"))),
+			      flowindentbox(0, 4,
+					    fragf(_("'_' to purge packages"))),
+			      flowindentbox(0, 4,
+					    fragf(_("'=' to place packages on hold"))),
+			      flowindentbox(0, 4,
+					    fragf(_("':' to keep packages in their current state without placing them on hold"))),
+			      flowindentbox(0, 4,
+					    fragf(_("'&M' to mark packages as automatically installed"))),
+			      flowindentbox(0, 4,
+					    fragf(_("'&m' to mark packages as manually installed")))));
+
+  out << _("Commands:") << endl;
+  out << f->layout(screen_width, screen_width, style());
+  delete f;
 }
 
 bool cmdline_do_prompt(bool as_upgrade,
@@ -705,7 +724,8 @@ bool cmdline_do_prompt(bool as_upgrade,
 		       bool showsize,
 		       bool always_prompt,
 		       int verbose,
-		       bool assume_yes)
+		       bool assume_yes,
+		       bool force_no_change)
 {
   bool cont=false;
   bool rval=true;
@@ -716,8 +736,23 @@ bool cmdline_do_prompt(bool as_upgrade,
       if(!cmdline_show_preview(true, to_install, to_hold, to_remove,
 			       showvers, showdeps, showsize, verbose) &&
 	 first &&
-	 !always_prompt)
+	 !always_prompt &&
+	 (*apt_cache_file)->BrokenCount()==0)
 	cont=true;
+      else if((*apt_cache_file)->BrokenCount() > 0)
+	{
+	  if(!cmdline_resolve_deps(to_install,
+				   to_hold,
+				   to_remove,
+				   to_purge,
+				   assume_yes,
+				   force_no_change,
+				   verbose))
+	    {
+	      cont=true;
+	      rval=false;
+	    }
+	}
       else if(assume_yes)
 	cont=true;
       else
@@ -727,19 +762,21 @@ bool cmdline_do_prompt(bool as_upgrade,
 	  while(!valid_response)
 	    {
 	      valid_response=true;
-	      printf(_("Do you want to continue? [Y/n/?] "));
 	      fflush(stdout);
 
-	      string response=prompt_string();
+	      string response=prompt_string(_("Do you want to continue? [Y/n/?] "));
 	      string::size_type loc=0;
 
 	      while(loc<response.size() && isspace(response[loc]))
 		++loc;
 
 	      if(loc==response.size())
-		response='y';
+		{
+		  response='y';
+		  loc=0;
+		}
 
-	      switch(toupper(response[0]))
+	      switch(toupper(response[loc]))
 		{
 		case 'Y':
 		  rval=true;
@@ -781,6 +818,7 @@ bool cmdline_do_prompt(bool as_upgrade,
 		case '-':
 		case '=':
 		case '_':
+		case ':':
 		  cmdline_parse_action(response, to_install, to_hold,
 				       to_remove, to_purge, verbose);
 		  break;
@@ -788,29 +826,7 @@ bool cmdline_do_prompt(bool as_upgrade,
 		  ui_preview();
 		case '?':
 		  valid_response=false;
-		  // TODO: use fragments instead.
-		  printf(_("Commands:\n"
-			   "  y: continue with the installation\n"
-			   "  n: abort and quit\n"
-			   "  i: show information about one or more packages;\n"
-			   "     the package names should follow 'i'\n"
-			   "  c: show the Debian changelogs of one or more packages;\n"
-			   "     the package names should follow 'c'\n"
-			   "  d: toggle display of dependency information\n"
-			   "  s: toggle display of changes in package sizes\n"
-			   "  v: toggle display of versions\n"
-			   "  e: enter the full visual interface\n"
-			   "\n"
-			   "  You may also specify modifications to the set of package actions.\n"
-			   "  To do so, type an action character followed by one or more package names\n"
-			   "or patterns.  The action will be applied to all packages. (you may specify\n"
-			   "additional actions; each will be applied to all packages that follow it)\n"
-			   "\n"
-			   "Actions:\n"
-			   "  + : Install\n"
-			   "  - : Remove\n"
-			   "  _ : Purge\n"
-			   "  = : Hold\n"));
+		  prompt_help(cout);
 		  break;
 		default:
 		  printf(_("Invalid response.  Please enter a valid command or '?' for help.\n"));
