@@ -115,6 +115,23 @@ static void setup_resolver(pkgset &to_install,
 	    }
 	}
     }
+
+  string dumpfile = aptcfg->Find(PACKAGE "::CmdLine::Resolver-Dump", "");
+  if(!dumpfile.empty())
+    {
+      ofstream f(dumpfile.c_str());
+      if(!f)
+	_error->Errno("dump_resolver", _("Unable to open %s for writing"), dumpfile.c_str());
+      else
+	{
+	  resman->dump(f);
+
+	  if(!f)
+	    _error->Errno("dump_resolver", _("Error writing resolver state to %s"), dumpfile.c_str());
+	  else
+	    cout << _("Resolver state successfully written!");
+	}
+    }
 }
 
 static inline fragment *flowindentbox(int i1, int irest, fragment *f)
@@ -189,7 +206,7 @@ static void resolver_help(ostream &out)
 // Given several versions with the same VerStr, choose one to output.
 static pkgCache::VerIterator choose_version(const vector<pkgCache::VerIterator> &choices)
 {
-  assert(!choices.empty());
+  eassert(!choices.empty());
 
   if(choices.size() == 1)
     return choices.front();
@@ -272,8 +289,8 @@ static void reject_or_mandate_version(const string &s,
 	  ver = aptitude_universe::version(pkg, choose_version(found),
 					   *apt_cache_file);
 
-	  assert(!ver.get_ver().end());
-	  assert(ver.get_pkg() == ver.get_ver().ParentPkg());
+	  eassert(!ver.get_ver().end());
+	  eassert(ver.get_pkg() == ver.get_ver().ParentPkg());
 	}
 
       if(is_reject)
@@ -599,7 +616,10 @@ bool cmdline_resolve_deps(pkgset &to_install,
 		bool done=false;
 		while(!done)
 		  {
-		    string response=prompt_string(_("No solution found within the allotted time.  Try harder? [Y/n]"));
+		    string response;
+
+		    if(!assume_yes)
+		      response = prompt_string(_("No solution found within the allotted time.  Try harder? [Y/n]"));
 
 		    string::size_type loc=0;
 		    while(loc<response.size() && isspace(response[loc]))
