@@ -23,13 +23,17 @@
  *  \param verbose the current verbosity level
  *  \param assume_yes if \b true, assume the user entered "yes"
  *                    at the prompt.
+ *  \param fornce_no_change if \b true, make an effort to avoid
+ *                          undoing the user's explicit requests as
+ *                          given in to_install et al.
  */
 
 int cmdline_simulate(bool as_upgrade,
 		     pkgset &to_install, pkgset &to_hold, pkgset &to_remove,
 		     pkgset &to_purge,
 		     bool showvers, bool showdeps, bool showsize,
-		     bool always_prompt, int verbose, bool assume_yes);
+		     bool always_prompt, int verbose,
+		     bool assume_yes, bool force_no_change);
 
 
 #endif // CMDLINE_SIMULATE_H
