@@ -33,6 +33,7 @@
 
 #include <generic/util/undo.h>
 #include <generic/util/util.h>
+#include <vscreen/transcode.h>
 
 #include <apt-pkg/configuration.h>
 #include <apt-pkg/depcache.h>
@@ -44,7 +45,7 @@
 
 #include <fstream>
 
-#include <assert.h>
+#include <generic/util/eassert.h>
 #include <signal.h>
 #include <unistd.h>
 #include <sys/stat.h>
@@ -456,7 +457,7 @@ static void surrounding_or_internal(const pkgCache::DepIterator &dep,
     }
 
   // If not, something is wrong with apt's cache.
-  assert(found);
+  eassert(found);
 }
 
 void surrounding_or(pkgCache::DepIterator dep,
@@ -601,11 +602,15 @@ bool package_trusted(const pkgCache::VerIterator &ver)
     {
       pkgIndexFile *index;
 
-      if(apt_source_list->FindIndex(i.File(), index) && !index->IsTrusted())
-	return false;
+      if(!apt_source_list->FindIndex(i.File(), index))
+	// Corresponds to the currently installed package, which is
+	// always "trusted".
+	return true;
+      else if(index->IsTrusted())
+	return true;
     }
 
-  return true;
+  return false;
 }
 
 /** \return \b true if d1 subsumes d2; that is, if one of the
@@ -845,3 +850,62 @@ bool is_interesting_dep(const pkgCache::DepIterator &d,
       abort();
     }
 }
+
+std::wstring get_short_description(const pkgCache::VerIterator &ver)
+{
+  if(ver.end() || ver.FileList().end() || apt_package_records == NULL)
+    return std::wstring();
+
+#ifndef HAVE_DDTP
+  pkgCache::VerFileIterator vf = ver.FileList();
+
+  if(vf.end())
+    return std::wstring();
+  else
+    return transcode(apt_package_records->Lookup(vf).ShortDesc(), "UTF-8");
+#else
+  pkgCache::DescIterator d = ver.TranslatedDescription();
+
+  if(d.end())
+    return std::wstring();
+
+  pkgCache::DescFileIterator df = d.FileList();
+
+  if(df.end())
+    return std::wstring();
+  else
+    // Since Packages files don't have a bundled description and it
+    // just feels icky for the data in Packages to depend on the
+    // encoding, I just force a sane encoding on it here.  Really,
+    // though, Packages files should have an encapsulated encoding
+    // somewhere.
+    return transcode(apt_package_records->Lookup(df).ShortDesc(), "UTF-8");
+#endif
+}
+
+std::wstring get_long_description(const pkgCache::VerIterator &ver)
+{
+  if(ver.end() || ver.FileList().end() || apt_package_records == NULL)
+    return std::wstring();
+
+#ifndef HAVE_DDTP
+  pkgCache::VerFileIterator vf = ver.FileList();
+
+  if(vf.end())
+    return std::wstring();
+  else
+    return transcode(apt_package_records->Lookup(vf).LongDesc(), "UTF-8");
+#else
+  pkgCache::DescIterator d = ver.TranslatedDescription();
+
+  if(d.end())
+    return std::wstring();
+
+  pkgCache::DescFileIterator df = d.FileList();
+
+  if(df.end())
+    return std::wstring();
+  else
+    return transcode(apt_package_records->Lookup(df).LongDesc(), "UTF-8");
+#endif
+}
