@@ -45,7 +45,7 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <errno.h>
-#include <assert.h>
+#include <generic/util/eassert.h>
 
 #include <sigc++/adaptors/bind.h>
 #include <sigc++/functors/mem_fun.h>
@@ -333,6 +333,8 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 
   new_package_count=0;
 
+  pre_package_state_changed();
+
   // Act on them
   for(pkgCache::PkgIterator i=PkgBegin(); !i.end(); i++)
     {
@@ -361,7 +363,10 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 	  if(!estate.candver.empty() && estate.upgrade)
 	    {
 	      for(pkgCache::VerIterator ver=i.VersionList(); !ver.end(); ++ver)
-		if(ver.VerStr()==estate.candver)
+		if(ver.VerStr()==estate.candver &&
+		   (ver.Downloadable() ||
+		    (ver == ver.ParentPkg().CurrentVer() &&
+		     ver.ParentPkg()->CurrentState != pkgCache::State::ConfigFiles)))
 		  SetCandidateVersion(ver);
 
 	      MarkInstall(i, false);
@@ -491,6 +496,7 @@ void aptitudeDepCache::mark_all_upgradable(bool with_autoinst,
 
 	  if(do_upgrade)
 	    {
+	      pre_package_state_changed();
 	      dirty = true;
 
 	      MarkInstall(i, do_autoinstall);
@@ -617,7 +623,7 @@ bool aptitudeDepCache::save_selection_list(OpProgress &prog,
 	      return false;
 	    }
 
-	  if(link(statefile.c_str(), oldstr.c_str()) != 0)
+	  if(link(statefile.c_str(), oldstr.c_str()) != 0 && errno != ENOENT)
 	    {
 	      _error->Errno("save_selection_list", _("failed to rename %s to %s"),
 			    statefile.c_str(), (statefile + ".old").c_str());
@@ -931,7 +937,10 @@ void aptitudeDepCache::set_candidate_version(const VerIterator &ver,
 
   dirty=true;
 
-  if(!ver.end())
+  if(!ver.end() &&
+     (ver.Downloadable() ||
+      (ver == ver.ParentPkg().CurrentVer() &&
+       ver.ParentPkg()->CurrentState != pkgCache::State::ConfigFiles)))
     {
       pre_package_state_changed();
       // Use the InstVerIter instead of GetCandidateVersion, since
@@ -1145,6 +1154,7 @@ bool aptitudeDepCache::try_fix_broken(undo_group *undo)
     }
 
   pkgProblemResolver fixer(this);
+  pre_package_state_changed();
   for(pkgCache::PkgIterator i=PkgBegin(); !i.end(); i++)
     {
       fixer.Clear(i);
@@ -1246,7 +1256,7 @@ void aptitudeDepCache::begin_action_group()
 
 void aptitudeDepCache::end_action_group(undo_group *undo)
 {
-  assert(group_level>0);
+  eassert(group_level>0);
 
   if(group_level==1)
     {
@@ -1312,7 +1322,9 @@ void aptitudeDepCache::restore_apt_state(const apt_state_snapshot *snapshot)
 void aptitudeDepCache::mark_package(const PkgIterator &pkg,
 				    const VerIterator &ver,
 				    bool follow_recommends,
-				    bool follow_suggests)
+				    bool follow_suggests,
+				    bool debug,
+				    int level)
 {
   pkgDepCache::StateCache &state=(*this)[pkg];
   aptitude_state &estate=get_ext_state(pkg);
@@ -1325,19 +1337,64 @@ void aptitudeDepCache::mark_package(const PkgIterator &pkg,
   if(state.Delete() && estate.remove_reason==unused)
     {
       if(ver==candver)
-	mark_install(pkg, false, false, NULL);
+	{
+	  if(debug)
+	    std::cout << string(level*2, ' ')
+		      << "Cancelling the deletion of "
+		      << pkg.Name() << " and installing its candidate version ("
+		      << ver.VerStr() << ")" << std::endl;
+
+	  mark_install(pkg, false, false, NULL);
+	}
       else if(ver==pkg.CurrentVer())
-	MarkKeep(pkg);
+	{
+	  if(debug)
+	    std::cout << string(level*2, ' ')
+		      << "Cancelling the deletion of "
+		      << pkg.Name() << " and reverting to its currently installed version ("
+		      << ver.VerStr() << ")" << std::endl;
+
+	  pre_package_state_changed();
+	  MarkKeep(pkg);
+	}
 
       instver=state.InstVerIter(*this);
     }
 
   if(!(ver==instver && !instver.end()))
-    return;
+    {
+      if(debug)
+	{
+	  std::cout << string(level*2, ' ')
+		    << "Not marking " << pkg.Name()
+		    << " " << (ver.end() ? "[UNINST]" : ver.VerStr())
+		    << ": ";
+
+	  if(ver == instver)
+	    std::cout << "it is the UNINST version";
+	  else
+	    std::cout << "it is not the version to be installed ("
+		      << (instver.end() ? "[UNINST]" : instver.VerStr())
+		      << ")" << std::endl;
+	}
+      return;
+    }
 
   if(estate.marked)
-    return;
+    {
+      if(debug)
+	std::cout << string(level*2, ' ')
+		  << "Not marking " << pkg.Name()
+		  << " " << (ver.end() ? "[UNINST]" : ver.VerStr())
+		  << ": it is already marked." << std::endl;
+      return;
+    }
 
+  if(debug)
+    std::cout << string(level*2, ' ')
+	      << "Marking " << pkg.Name() << " "
+	      << (ver.end() ? "[UNINST]" : ver.VerStr())
+	      << std::endl;
   estate.marked=true;
 
   if(!ver.end())
@@ -1351,17 +1408,35 @@ void aptitudeDepCache::mark_package(const PkgIterator &pkg,
 	     (follow_suggests &&
 	      d->Type==pkgCache::Dep::Suggests))
 	    {
+	      if(debug)
+		{
+		  std::cout << string(level*2, ' ')
+			    << "Following dependency "
+			    << d.ParentPkg().Name()
+			    << " "
+			    << d.DepType()
+			    << " "
+			    << d.TargetPkg().Name();
+		  if(d.TargetVer() != NULL)
+		    std::cout << " ("
+			      << d.CompType()
+			      << " "
+			      << d.TargetVer()
+			      << ")";
+		  std::cout << std::endl;
+		}
+
 	      // Try all versions of this package.
 	      for(VerIterator V=d.TargetPkg().VersionList(); !V.end(); ++V)
 		if(_system->VS->CheckDep(V.VerStr(), d->CompareOp, d.TargetVer()))
 		  mark_package(V.ParentPkg(), V,
-			       follow_recommends, follow_suggests);
+			       follow_recommends, follow_suggests, debug, level + 1);
 
 	      // Now try virtual packages
 	      for(PrvIterator prv=d.TargetPkg().ProvidesList(); !prv.end(); ++prv)
 		if(_system->VS->CheckDep(prv.ProvideVersion(), d->CompareOp, d.TargetVer()))
 		  mark_package(prv.OwnerPkg(), prv.OwnerVer(),
-			       follow_recommends, follow_suggests);
+			       follow_recommends, follow_suggests, debug, level + 1);
 	    }
 	}
     }
@@ -1394,6 +1469,10 @@ void aptitudeDepCache::mark_and_sweep(undo_group *undo)
     aptcfg->FindB(PACKAGE "::Keep-Recommends", false);
   bool follow_suggests=aptcfg->FindB(PACKAGE "::Keep-Suggests", false) ||
     aptcfg->FindB(PACKAGE "::Suggests-Important", false);
+  bool debug = aptcfg->FindB(PACKAGE "::GC-Debug", false);
+
+  if(debug)
+    std::cout << "*** BEGIN MARK AND SWEEP ***" << std::endl;
 
   for(pkgCache::PkgIterator p=PkgBegin(); !p.end(); ++p)
     {
@@ -1407,10 +1486,10 @@ void aptitudeDepCache::mark_and_sweep(undo_group *undo)
 	{
 	  if(PkgState[p->ID].Keep() && !p.CurrentVer().end())
 	    mark_package(p, p.CurrentVer(),
-			 follow_recommends, follow_suggests);
+			 follow_recommends, follow_suggests, debug, 0);
 	  else if(PkgState[p->ID].Install())
 	    mark_package(p, PkgState[p->ID].InstVerIter(GetCache()),
-			 follow_recommends, follow_suggests);
+			 follow_recommends, follow_suggests, debug, 0);
 	}
     }
 
@@ -1431,6 +1510,9 @@ void aptitudeDepCache::mark_and_sweep(undo_group *undo)
 	    {
 	      bool do_delete=true;
 
+	      if(debug)
+		std::cout << "*** Removing unused package " << p.Name() << std::endl;
+
 	      if(do_delete)
 		mark_delete(p,
 			    aptcfg->FindB(PACKAGE "::Purge-Unused", false),
@@ -1444,6 +1526,9 @@ void aptitudeDepCache::mark_and_sweep(undo_group *undo)
 
   delete matcher;
 
+  if(debug)
+    std::cout << "*** END MARK AND SWEEP ***" << std::endl;
+
   // end_action_group is written carefully so it works here:
   end_action_group(undo);
 }
