@@ -39,6 +39,7 @@
 #include <aptitude.h>
 
 #include <generic/util/util.h>
+#include <vscreen/transcode.h>
 
 #include <set>
 
@@ -50,7 +51,7 @@
 #include <apt-pkg/pkgsystem.h>
 #include <apt-pkg/version.h>
 
-#include <assert.h>
+#include <generic/util/eassert.h>
 #include <ctype.h>
 #include <stdio.h>
 #include <regex.h>
@@ -349,7 +350,7 @@ public:
     if(ver.end())
       return match_target(false, "");
     else
-      return match_target(true, apt_package_records->Lookup(ver.FileList()).LongDesc().c_str());
+      return match_target(true, transcode(get_long_description(ver).c_str()));
   }
 };
 
@@ -1046,7 +1047,7 @@ public:
   bool matches(const pkgCache::PkgIterator &pkg,
 	       const pkgCache::VerIterator &ver)
   {
-    assert(!pkg.end());
+    eassert(!pkg.end());
     if(ver.end())
       return false;
 
@@ -1085,7 +1086,7 @@ public:
   pkg_match_result *get_match(const pkgCache::PkgIterator &pkg,
 			      const pkgCache::VerIterator &ver)
   {
-    assert(!pkg.end());
+    eassert(!pkg.end());
     if(ver.end())
       return NULL;
 
@@ -1919,7 +1920,7 @@ std::string parse_substr(string::const_iterator &start,
 	  if(start == end)
 	    throw CompilationException(_("Unterminated literal string after %s"), rval.c_str());
 
-	  assert(*start == '"');
+	  eassert(*start == '"');
 	  ++start;
 	}
 
