@@ -20,22 +20,27 @@
 // This file defines a class to represent immutable sets.  These sets
 // behave like std::set, except that their contents cannot be changed;
 // they have no erase() or insert() operators, and only
-// const_iterators.  This restriction allows immutable sets to be
+// const_iterators [0].  This restriction allows immutable sets to be
 // implemented in a way that makes both copying and creating a new set
 // by adding an element very efficient (O(1) and O(lg n) respectively;
 // with std::set these are both O(n)).
+//
+// [0] this refers to the wtree_node class; imm::set is a wrapper
+// around a wtree_node pointer that emulates a mutative interface by
+// creating a new tree and modifying its encapsulated pointer to point
+// there instead.
 
 #ifndef IMMSET_H
 #define IMMSET_H
 
-#include <assert.h>
+#include <generic/util/eassert.h>
 #include <iostream>
 
 #include <vector>
 
 namespace imm
 {
-  /** A generic node in a weighted tree a described in "Implementing
+  /** A generic node in a weighted tree as described in "Implementing
    *  Sets Efficiently In A Functional Language".  The tree invariant
    *  is that the tree is not "too unbalanced"; in this case, that no
    *  subtree is more than 4 times larger than its sibling.  (note
@@ -43,8 +48,7 @@ namespace imm
    *
    *  A brief note on choice of algorithm: while a rbtree is more
    *  common and may be slightly more efficient, it is a much trickier
-   *  data structure to implement, and some operations (like set-union
-   *  and set-intersection) may be difficult to implement efficiently.
+   *  data structure to implement.
    *
    *  The weighted-tree data structure is reasonably efficient and
    *  much more straightforward to implement correctly.
@@ -99,14 +103,14 @@ namespace imm
 
       void incref() const
       {
-	assert(refcount>0);
+	eassert(refcount>0);
 
 	++refcount;
       }
 
       void decref() const
       {
-	assert(refcount>0);
+	eassert(refcount>0);
 	--refcount;
 
 	if(refcount == 0)
@@ -442,7 +446,7 @@ namespace imm
 
       const_iterator &operator++()
       {
-	assert(!path.empty());
+	eassert(!path.empty());
 
 	if(!path.back().first)
 	  {
@@ -539,7 +543,12 @@ namespace imm
       return node(tmp.second, l, tmp.first);
     }
 
-    /** \return \b true if n1 and n2 share an element. */
+    /** \return \b true if there exist elements x1 and x2 in n1 and n2
+     *  respectively such that x1 and x2 are equivalent and f(x1, x2)
+     *  holds.  If f is \b lambda x1 x2 . \b true, then this routine
+     *  simply tests whether the sets represented by n1 and n2
+     *  intersect.
+     */
     template<typename F>
     bool nodes_intersect(const node &n1, const node &n2, const F &f) const
     {
@@ -570,7 +579,11 @@ namespace imm
           nodes_intersect(n1.getRight(), n2.getRight(), f);
     }
 
-    /** \return \b true if n1 contains n2 under f. */
+    /** \return \b true if n1 contains n2 under f; i.e., if for each
+     *  element x2 of n2 there exists an element x1 of n1 such that x1
+     *  is equivalent to x2 and f(x1, x2) holds.  If f is
+     *  \b lambda x1 x2 . \b true, then this is simply set containment.
+     */
     template<typename F>
     bool node_contains(const node &n1, const node &n2, const F &f) const
     {
@@ -626,6 +639,7 @@ namespace imm
     {
     }
 
+    /** The binary predicate \b lambda x1 x2 . \b true */
     template<typename T>
     struct universal_relation
     {
@@ -649,16 +663,16 @@ namespace imm
     }
 
     /** Insert an element into a tree, returning a new tree.  This is
-     *  not a member function, to stress that it does NOT modify the
-     *  old tree; instead, it returns a new tree containing the
-     *  element in addition to the elements of the old tree.
+     *  a static function to stress that it does NOT modify the old
+     *  tree; instead, it returns a new tree containing the element in
+     *  addition to the elements of the old tree.
      */
     static set add(const set &old, const Val &x)
     {
       return set(old.add(old.root, x), old.value_compare);
     }
 
-    /** Like insert, but updates existing equivalent elements. */
+    /** Like add, but updates existing equivalent elements. */
     static set addUpdate(const set &old, const Val &x)
     {
       return set(old.addUpdate(old.root, x), old.value_compare);
