@@ -4,10 +4,6 @@
 //
 //  Various routines dealing with messages about package trust.
 
-#include <apt-pkg/init.h> // For the APT version.
-
-#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
-
 #ifndef TRUST_H
 #define TRUST_H
 
@@ -24,5 +20,3 @@ class fragment;
 fragment *make_untrusted_warning(const pkgCache::VerIterator &ver);
 
 #endif
-
-#endif
