@@ -363,7 +363,7 @@ static bool do_read_only_permission()
 
 static void do_read_only_fail()
 {
-  assert(active_download);
+  eassert(active_download);
 
   show_message(_("You may not modify the state of any package while a download is underway."));
 }
@@ -870,7 +870,8 @@ static void do_help_help()
 
   vs_widget_ref w=vs_dialog_fileview(buf, NULL,
 				     arg(sigc::ptr_fun(pager_search)),
-				     arg(sigc::ptr_fun(pager_repeat_search)));
+				     arg(sigc::ptr_fun(pager_repeat_search)),
+				     encoding);
   w->show_all();
 
   popup_widget(w);
@@ -1133,7 +1134,7 @@ static void do_show_preview()
 {
   if(!active_preview_tree.valid())
     {
-      assert(!active_preview.valid());
+      eassert(!active_preview.valid());
 
       pkg_grouppolicy_factory *grp=NULL;
       std::string grpstr;
@@ -1181,7 +1182,7 @@ static void do_show_preview()
     }
   else
     {
-      assert(active_preview.valid());
+      eassert(active_preview.valid());
       active_preview->show();
     }
 }
@@ -1207,16 +1208,19 @@ static void auto_fix_broken()
 
   try
     {
-      assert(resman != NULL);
-      assert(resman->resolver_exists());
+      eassert(resman != NULL);
+      eassert(resman->resolver_exists());
 
       aptitude_solution sol = resman->get_solution(resman->get_selected_solution(), 0);
 
       (*apt_cache_file)->apply_solution(sol, undo);
-      show_message(fragf("%s%n%n%F",
-			 _("Some packages were broken and have been fixed:"),
-			 solution_fragment(sol)),
-		   NULL);
+
+      vs_widget_ref d = vs_dialog_ok(fragf("%s%n%n%F",
+					   _("Some packages were broken and have been fixed:"),
+					   solution_fragment(sol)),
+				     NULL);
+
+      popup_widget(d, true);
     }
   catch(NoMoreSolutions)
     {
@@ -1563,7 +1567,7 @@ static void do_reload_cache()
 }
 #endif
 
-static void start_solution_calculation();
+static void start_solution_calculation(bool blocking = true);
 
 class interactive_continuation : public resolver_manager::background_continuation
 {
@@ -1626,7 +1630,7 @@ public:
 
   void no_more_time()
   {
-    start_solution_calculation();
+    start_solution_calculation(false);
   }
 
   void interrupted()
@@ -1635,23 +1639,35 @@ public:
 };
 
 // If the current solution pointer is past the end of the solution
-// list, queue up a calculation for it in the background thread.
-static void start_solution_calculation()
+// list, queue up a calculation for it in the background thread.  If
+// "blocking" is true, then the calling thread will wait for the
+// background resolver thread to finish its calculation before
+// returning from this function.  IF THE CALLING THREAD IS THE
+// BACKGROUND THREAD THIS WILL DEADLOCK IF BLOCKING IS TRUE! (see above for such a case)
+static void start_solution_calculation(bool blocking)
 {
   resolver_manager::state state = resman->state_snapshot();
 
   if(state.resolver_exists &&
      state.selected_solution == state.generated_solutions &&
+     !state.solutions_exhausted &&
      !state.background_thread_active)
-    resman->get_solution_background_blocking(resman->get_selected_solution(),
-					     aptcfg->FindI(PACKAGE "::ProblemResolver::StepLimit", 5000),
-					     aptcfg->FindI(PACKAGE "::ProblemResolver::WaitSteps", 50),
-					     new interactive_continuation(resman));
+    {
+      const int selected = state.selected_solution;
+      const int limit = aptcfg->FindI(PACKAGE "::ProblemResolver::StepLimit", 5000);
+      const int wait_steps = aptcfg->FindI(PACKAGE "::ProblemResolver::WaitSteps", 50);
+      interactive_continuation * const k = new interactive_continuation(resman);
+
+      if(blocking)
+	resman->get_solution_background_blocking(selected, limit, wait_steps, k);
+      else
+	resman->get_solution_background(selected, limit, k);
+    }
 }
 
 static void do_connect_resolver_callback()
 {
-  resman->state_changed.connect(sigc::ptr_fun(&start_solution_calculation));
+  resman->state_changed.connect(sigc::bind(sigc::ptr_fun(&start_solution_calculation), true));
   // We may have missed a signal before making the connection:
   start_solution_calculation();
   resman->state_changed.connect(sigc::ptr_fun(&vscreen_update));
