@@ -37,7 +37,7 @@ void vs_progress::paint(const style &st)
 
   if(!Op.empty())
     {
-      assert(Percent>=0 && Percent<=100);
+      eassert(Percent>=0 && Percent<=100);
       barsize=int(Percent*width/100.0);
 
       snprintf(percent_string, 50, ": %i%%", int(Percent));
