@@ -26,15 +26,34 @@ private:
    *  \return the slider location, or -1 if it is not visible.
    */
   int get_slider();
-public:
+protected:
   vs_scrollbar(direction _dir, int _val, int _max)
     :dir(_dir), max(_max), val(_val) {}
 
   vs_scrollbar(direction _dir)
     :dir(_dir), max(0), val(0) {}
+public:
+  static
+  ref_ptr<vs_scrollbar> create(direction dir, int val, int max)
+  {
+    ref_ptr<vs_scrollbar> rval(new vs_scrollbar(dir, val, max));
+    rval->decref();
+    return rval;
+  }
+
+  static
+  ref_ptr<vs_scrollbar> create(direction dir)
+  {
+    ref_ptr<vs_scrollbar> rval(new vs_scrollbar(dir));
+    rval->decref();
+    return rval;
+  }
+
+  void paint(const style &st);
+
+  int width_request();
+  int height_request(int w);
 
-  void paint();
-  size size_request();
   bool get_cursorvisible();
   point get_cursorloc();
   void dispatch_mouse(short id, int x, int y, int z, mmask_t bstate);
@@ -45,7 +64,9 @@ public:
    *  via the scrollbar.  Its argument is \b true for a "page up" and
    *  \b false for a "page down".
    */
-  SigC::Signal1<void, bool> scrollbar_interaction;
+  sigc::signal1<void, bool> scrollbar_interaction;
 };
 
+typedef ref_ptr<vs_scrollbar> vs_scrollbar_ref;
+
 #endif
