@@ -4,23 +4,26 @@
 // order (it is possible to change the stacking order)
 //
 //  The size of the widget is unrelated to the sizes of its components.
+//  (why? why not size it in a more flexible way?)
 
 #ifndef VS_STACKED_H
 #define VS_STACKED_H
 
 #include "vs_passthrough.h"
 
+#include <sigc++/connection.h>
+
 class vs_stacked:public vs_passthrough
 {
   // bleach, but we need somewhere to store the info on what the signals to
   // disconnect are :(
   struct child_info
   {
-    vscreen_widget *w;
+    vs_widget_ref w;
 
-    SigC::Connection shown_conn, hidden_conn;
+    sigc::connection shown_conn, hidden_conn;
 
-    child_info(vscreen_widget *_w,
+    child_info(const vs_widget_ref &_w,
 	       SigC::Connection &_shown_conn,
 	       SigC::Connection &_hidden_conn)
       :w(_w), shown_conn(_shown_conn),
@@ -39,24 +42,47 @@ class vs_stacked:public vs_passthrough
 
   void hide_widget();
 protected:
-  void paint();
-public:
+  void paint(const style &st);
+
   // The size passed in is used as a preferred size.  (what we get might be
   // larger or smaller)
-  vs_stacked(int w=0, int h=0);
+  vs_stacked(int w, int h);
+public:
+  ~vs_stacked();
 
-  void add_widget(vscreen_widget *w);
-  void rem_widget(vscreen_widget *w);
-  void raise_widget(vscreen_widget *w);
-  void lower_widget(vscreen_widget *w);
+  void destroy();
+
+  static ref_ptr<vs_stacked> create(int w=0, int h=0)
+  {
+    ref_ptr<vs_stacked> rval(new vs_stacked(w, h));
+    rval->decref();
+    return rval;
+  }
+
+  void add_widget(const vs_widget_ref &w);
+  void rem_widget(const vs_widget_ref &w);
+  void raise_widget(const vs_widget_ref &w);
+  void lower_widget(const vs_widget_ref &w);
+
+  void raise_widget_bare(vscreen_widget &w)
+  {
+    raise_widget(vs_widget_ref(&w));
+  }
+  void lower_widget_bare(vscreen_widget &w)
+  {
+    lower_widget(vs_widget_ref(&w));
+  }
 
   void dispatch_mouse(short id, int x, int y, int z, mmask_t bstate);
 
-  vscreen_widget *vs_stacked::get_focus();
+  vs_widget_ref vs_stacked::get_focus();
 
   void show_all();
 
-  size size_request();
+  int width_request();
+  int height_request(int w);
 };
 
+typedef ref_ptr<vs_stacked> vs_stacked_ref;
+
 #endif
