@@ -50,7 +50,7 @@
 
 #include <signal.h>
 
-#include <assert.h>
+#include <generic/util/eassert.h>
 #include <sys/time.h>
 
 #include <map>
@@ -966,11 +966,10 @@ void vscreen_suspend()
   suspended_with_signals = true;
 
   struct sigaction act;
+
+  memset(&act,0,sizeof(act));
   act.sa_handler = SIG_IGN;
-  act.sa_sigaction = 0;
   sigemptyset(&act.sa_mask);
-  act.sa_flags = 0;
-  act.sa_restorer = 0;
 
   sigaction(SIGCONT, &act, &oldsigcont);
   sigaction(SIGTSTP, &act, &oldsigtstp);
@@ -980,7 +979,8 @@ void vscreen_suspend()
 
 void vscreen_shutdown()
 {
-  toplevel->destroy();
+  if(toplevel.valid())
+    toplevel->destroy();
   toplevel = NULL;
 
   vscreen_suspend();
